# $Id: repec.py 3015 2010-02-07 23:24:56Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-8.1.0/src/repec.py $
import os
from cqlparser import parseString as parseCQL
from datetime import date 
from lxml.etree import parse, tostring, XSLT
from merescocore.components.http import utils as httputils
from merescocore.framework import Observable
from StringIO import StringIO
from xml.sax.saxutils import escape as xml_escape
from urlparse import urlsplit

# FIXME nsMap uit config gebruiken
nsMap = {
    'ddi'   :"http://www.icpsr.umich.edu/DDI",
    'dii'   :"urn:mpeg:mpeg21:2002:01-DII-NS",
    'didl'  :"urn:mpeg:mpeg21:2002:02-DIDL-NS",
    'dip'   :"urn:mpeg:mpeg21:2005:01-DIP-NS",
    'foaf'  :"http://xmlns.com/foaf/0.1/",
    'mods'  :"http://www.loc.gov/mods/v3",
    'nereus':"http://www.nereus4economics.info/",
    'rdf'   :"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
    'extra' :"http://meresco.com/namespace/fields/extra",
    'meta'  :"http://meresco.com/namespace/harvester/meta",
}

# FIXME naar config
# series -> partner
seriesMap = {
    'carlos':"uc3m-es",
    'cergez':"cerge-cuni-cz",
    'columb':"columbia-university-us",
    'dauphi':"dauphine-fr",
    'euiflo':"eui-eu",
    'ifwkie':"ifw-kiel-de",
    'leuven':"kuleuven-be",
    'louvai':"uclouvain-be",
    'lselon':"lse-ac-uk",
    'maastr':"unimaas-nl",
    'oxford':"ox-ac-uk",
    'sciepo':"www-sciences-po-fr",
    'tilbur':"tilburguniversity-nl",
    'ucddub':"ucd-ie",
    'toulou':"univ-tlse1-fr",
    'warwic':"Warwick-uk",
}

# FIXME naar config
# dit wordt nog niet gebruikt
# partner weglaten uit seriesMap is voldoende
noSeries = {
    'eurrot':"repub-eur-nl",
    'ulbbru':"ulb-ac-be",
    'ucllon':"ucl-ac-uk",
}

# FIXME naar config
excludeSet = {
    'ifwkie':"hdl_10419_29",
}

# FIXME dit moet uit de admin files gehaald worden
setMap = {
    'cerge-cuni-cz'         :"",
    'columbia-university-us':"",
    'dauphine-fr'           :"",
    'eui-eu'                :"",
    'ifw-kiel-de'           :"",
    'kuleuven-be'           :"",
    'lse-ac-uk'             :"", # fulltext
    'ox-ac-uk'              :"",
    'repub-eur-nl'          :"",
    'tilburguniversity-nl'  :"",
    'uc3m-es'               :"",
    'ucd-ie'                :"",
    'ucl-ac-uk'             :"", # fulltext
    'uclouvain-be'          :"",
    'ulb-ac-be'             :"",
    'unimaas-nl'            :"withfulltext",
    'univ-tlse1-fr'         :"",
    'Warwick-uk'            :"",
    'www-sciences-po-fr'    :""
}

def okTextPlain():
    CRLF = "\r\n"
    ContentTypeTextPlain = "Content-Type: text/plain"
    Ok = "HTTP/1.0 200 Ok" + CRLF
    okTextPlain = "HTTP/1.0 200 OK" + CRLF + \
             ContentTypeTextPlain + CRLF + \
             CRLF
    return okTextPlain

class RepecRequest(Observable):
    def __init__(self, configs, xsltFileList):
        Observable.__init__(self)
        self.configs = configs
        s = ""
        for file in xsltFileList:
            if file.find("didlmods2redif") > -1: s = file
        self._xslt = XSLT(parse(open(s)))
        self._docroot = ""
        self._baseURL =""
        self._maximumRecords = ""
        self.serverConfig  = self.configs ['server']
        if self.serverConfig.get('enabled', section='repec', default = False) == 'True':
            self._docroot = self.serverConfig.get('docroot', section='repec')
            self._baseURL = self.serverConfig.get('baseURL', section='repec')
            self._maximumRecords = int(self.serverConfig.get('maximumRecords', section='repec'))

    def remove_files_from_directories(self, top):
        tree = os.walk(top)
        for directory in tree:
            path = directory[0]
            if path == top: continue
            for file in directory[2]:
                os.remove(os.path.join(path,file))

    def make_file(self, query, seriesId):
        #print "query=" + query
        parsetree = parseCQL(query)
        start = 0
        stop = self._maximumRecords
        sortBy =  ''
        sortDescending = True
        total, recordIds = self.any.executeCQL(cqlAbstractSyntaxTree=parsetree, start=start, stop=stop, sortBy=sortBy, sortDescending=sortDescending)
        #s = "total=" + str(total) + " query=" + query
        s = "\xEF\xBB\xBF"
        diag = ""
        for recordId in recordIds:
            try:
                metadata = parse(self.any.getStream(recordId, 'metadata'))
            except:
                diag += "There was a problem with retrieving metadata.\n"
                continue
            didlMods =  metadata.xpath("didl:DIDL", nsMap)[0]
            try:
                docid = didlMods.xpath("didl:Item/didl:Descriptor/didl:Statement/dii:Identifier/text()", nsMap)[0].split("#", 1)[0]
            except IndexError:
                continue
            handle = "RePEc:ner:" + seriesId + ":" + docid
            qhandle = "\'" + handle + "\'"
            redif = self._xslt(didlMods, handle=qhandle)
            s += str(redif)
            s += "\n\n"
            # FIXME handle wegschrijven naar storage
            #meta = parse(self.any.getStream(recordId, 'meta'))
            #qtoday = "\'" + str(date.today()) + "\'"
            #newMeta = xml_escape(tostring(self._addHandle(meta, handle=qhandle, today=qtoday)))
            #doc = parse(StringIO("""<document xmlns="http://meresco.com/namespace/harvester/document"><part name="meta">%(meta)s</part></document>""" % {'meta':newMeta}))
            #self.do.add(recordId, "meta", doc)
        return total, s, diag

    def handleRequest(self, RequestURI='', **kwargs):
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        path = Path.split('/')
        # path[1] == 'repec'
        if self.serverConfig.get('enabled', section='repec') != 'True':
            yield okTextPlain()
            yield "This is not a RePEc archive generator.\n"
        elif len(path)==2 or not path[2]:
            yield okTextPlain()
            yield "Specify archive in path; at this moment I know ner, ner and ner.\n"
        else:
            archive = path[2]
            if archive == "ner":
                # archive file; not generated; static file in $docroot/ner/
                """Template-type: ReDIF-Archive 1.0
Handle: RePEc:ner
Name: EconomistsOnline Archive
Maintainer-Email: t.w.place@uvt.nl
URL: https://bzv.place.pukurin.uvt.nl/repec/nrs"""
                # get partner information
                partners = {}
                for key in seriesMap:
                    id = seriesMap[key]
                    url = self._baseURL + "?request=getPartner&partnerId=" + id
                    #print "url=" + url
                    partnerInfo = parse(url)
                    status = partnerInfo.xpath('properties/entry[@key="neeo.partner.status"]/text()')[0]
                    if status == "new": continue
                    #yield tostring(partnerInfo)
                    institutionPath = '/nereus:partner/rdf:RDF/foaf:Organization[rdf:type="info:eu-repo/semantics/institution"]'
                    info = {}
                    info['Provider-Name']     = partnerInfo.xpath(institutionPath+"/foaf:name/text()",                       nsMap)[0]
                    info['Provider-Homepage'] = partnerInfo.xpath(institutionPath+"/foaf:homepage/@rdf:resource",            nsMap)[0]
                    info['Maintainer-Name']   = partnerInfo.xpath(institutionPath+"/nereus:contacts/nereus:contact/@name",   nsMap)[0]
                    info['Maintainer-Email']  = partnerInfo.xpath(institutionPath+"/nereus:contacts/nereus:contact/@e-mail", nsMap)[0]
                    #print info['Provider-Name'].encode('utf-8')
                    #print info['Provider-Homepage']
                    #print info['Maintainer-Name'].encode('utf-8')
                    #print info['Maintainer-Email']
                    partners[key] = info
                # series file
                seriesFile = open(self._docroot+"/"+archive+"/"+archive+"seri.rdf", 'w')
                yield okTextPlain()
                for key in partners:
                    s = """Template-type: ReDIF-Series 1.0
Name: Open Access publications from %(organisation)s
Provider-Name: %(organisation)s
Provider-Homepage: %(homepage)s
Maintainer-Name: %(name)s
Maintainer-Email: %(email)s
Handle: RePEc:ner:%(seriesid)s """ %{'organisation':partners[key]['Provider-Name'],
                                     'homepage'    :partners[key]['Provider-Homepage'],
                                     'name'        :partners[key]['Maintainer-Name'],
                                     'email'       :partners[key]['Maintainer-Email'],
                                     'seriesid'    :key}
                    s += "\n\n"
                    #yield s
                    seriesFile.write(s)
                    self.remove_files_from_directories(os.path.join(self._docroot, archive))
                for series in seriesMap:
                    partnerId = seriesMap[series]
                    #print "series="+series
                    #print "partnerId="+partnerId
                    path = self._docroot+"/"+archive+"/"+series+"/"
                    print "path="+path
                    if not os.path.exists(path):
                        os.mkdir(path)
                    query = partnerId + ' and fulltext.url.exists exact true not mods.genre="info:eu-repo/semantics/workingpaper"'
                    if series in excludeSet:
                        query += ' not set=' + excludeSet[series]
                    yield "query="+query+"\n"
                    total, result, diag = self.make_file(query, series)
                    yield "total=" + str(total) + "\n" + diag + "\n"
                    if total > 0:
                        file = open(path+"metadata.rdf", 'w')
                        file.write(result)
                        file.close()
                yield "\n-- finished --"
